@@ -24,17 +24,13 @@
 const {
   Array,
   ArrayIsArray,
-  ArrayPrototypeJoin,
   MathFloor,
-  NumberPrototypeToString,
   ObjectDefineProperty,
+  ObjectHasOwn,
   ObjectKeys,
-  ObjectPrototypeHasOwnProperty,
   ObjectSetPrototypeOf,
   ObjectValues,
-  RegExpPrototypeExec,
   SafeSet,
-  StringPrototypeToLowerCase,
   Symbol,
 } = primordials;
 
@@ -99,11 +95,11 @@ const RE_CONN_CLOSE = /(?:^|\W)close(?:$|\W)/i;
 // against the word "cookie." As of V8 6.6 this is faster than handrolling or
 // using a case-insensitive RegExp.
 function isCookieField(s) {
-  return s.length === 6 && StringPrototypeToLowerCase(s) === 'cookie';
+  return s.length === 6 && s.toLowerCase() === 'cookie';
 }
 
 function isContentDispositionField(s) {
-  return s.length === 19 && StringPrototypeToLowerCase(s) === 'content-disposition';
+  return s.length === 19 && s.toLowerCase() === 'content-disposition';
 }
 
 function OutgoingMessage(options) {
@@ -230,7 +226,7 @@ ObjectDefineProperty(OutgoingMessage.prototype, '_headers', {
       // Refs: https://github.com/nodejs/node/pull/30958
       for (let i = 0; i < keys.length; ++i) {
         const name = keys[i];
-        headers[StringPrototypeToLowerCase(name)] = [name, val[name]];
+        headers[name.toLowerCase()] = [name, val[name]];
       }
     }
   }, 'OutgoingMessage.prototype._headers is deprecated', 'DEP0066'),
@@ -339,7 +335,7 @@ OutgoingMessage.prototype.uncork = function() {
   assert(this.chunkedEncoding);
 
   let callbacks;
-  this._send(NumberPrototypeToString(len, 16), 'latin1', null);
+  this._send(len.toString(16), 'latin1', null);
   this._send(crlf_buf, null, null);
   for (let n = 0; n < buf.length; n += 3) {
     this._send(buf[n + 0], buf[n + 1], null);
@@ -491,7 +487,7 @@ function _storeHeader(firstLine, headers) {
       }
     } else {
       for (const key in headers) {
-        if (ObjectPrototypeHasOwnProperty(headers, key)) {
+        if (ObjectHasOwn(headers, key)) {
           processHeader(this, state, key, headers[key], true);
         }
       }
@@ -613,7 +609,7 @@ function processHeader(self, state, key, value, validate) {
   if (ArrayIsArray(value)) {
     if (
       (value.length < 2 || !isCookieField(key)) &&
-      (!self[kUniqueHeaders] || !self[kUniqueHeaders].has(StringPrototypeToLowerCase(key)))
+      (!self[kUniqueHeaders] || !self[kUniqueHeaders].has(key.toLowerCase()))
     ) {
       // Retain for(;;) loop for performance reasons
       // Refs: https://github.com/nodejs/node/pull/30958
@@ -621,7 +617,7 @@ function processHeader(self, state, key, value, validate) {
         storeHeader(self, state, key, value[i], validate);
       return;
     }
-    value = ArrayPrototypeJoin(value, '; ');
+    value = value.join('; ');
   }
   storeHeader(self, state, key, value, validate);
 }
@@ -636,12 +632,12 @@ function storeHeader(self, state, key, value, validate) {
 function matchHeader(self, state, field, value) {
   if (field.length < 4 || field.length > 17)
     return;
-  field = StringPrototypeToLowerCase(field);
+  field = field.toLowerCase();
   switch (field) {
     case 'connection':
       state.connection = true;
       self._removedConnection = false;
-      if (RegExpPrototypeExec(RE_CONN_CLOSE, value) !== null)
+      if (RE_CONN_CLOSE.test(value))
         self._last = true;
       else
         self.shouldKeepAlive = true;
@@ -649,7 +645,7 @@ function matchHeader(self, state, field, value) {
     case 'transfer-encoding':
       state.te = true;
       self._removedTE = false;
-      if (RegExpPrototypeExec(RE_TE_CHUNKED, value) !== null)
+      if (RE_TE_CHUNKED.test(value))
         self.chunkedEncoding = true;
       break;
     case 'content-length':
@@ -692,7 +688,7 @@ function parseUniqueHeadersOption(headers) {
   const unique = new SafeSet();
   const l = headers.length;
   for (let i = 0; i < l; i++) {
-    unique.add(StringPrototypeToLowerCase(headers[i]));
+    unique.add(headers[i].toLowerCase());
   }
 
   return unique;
@@ -709,7 +705,7 @@ OutgoingMessage.prototype.setHeader = function setHeader(name, value) {
   if (headers === null)
     this[kOutHeaders] = headers = { __proto__: null };
 
-  headers[StringPrototypeToLowerCase(name)] = [name, value];
+  headers[name.toLowerCase()] = [name, value];
   return this;
 };
 
@@ -761,7 +757,7 @@ OutgoingMessage.prototype.appendHeader = function appendHeader(name, value) {
   validateHeaderName(name);
   validateHeaderValue(name, value);
 
-  const field = StringPrototypeToLowerCase(name);
+  const field = name.toLowerCase();
   const headers = this[kOutHeaders];
   if (headers === null || !headers[field]) {
     return this.setHeader(name, value);
@@ -792,7 +788,7 @@ OutgoingMessage.prototype.getHeader = function getHeader(name) {
   if (headers === null)
     return;
 
-  const entry = headers[StringPrototypeToLowerCase(name)];
+  const entry = headers[name.toLowerCase()];
   return entry && entry[1];
 };
 
@@ -841,7 +837,7 @@ OutgoingMessage.prototype.getHeaders = function getHeaders() {
 OutgoingMessage.prototype.hasHeader = function hasHeader(name) {
   validateString(name, 'name');
   return this[kOutHeaders] !== null &&
-    !!this[kOutHeaders][StringPrototypeToLowerCase(name)];
+    !!this[kOutHeaders][name.toLowerCase()];
 };
 
 
@@ -852,7 +848,7 @@ OutgoingMessage.prototype.removeHeader = function removeHeader(name) {
     throw new ERR_HTTP_HEADERS_SENT('remove');
   }
 
-  const key = StringPrototypeToLowerCase(name);
+  const key = name.toLowerCase();
 
   switch (key) {
     case 'connection':
@@ -1013,7 +1009,7 @@ function write_(msg, chunk, encoding, callback, fromEnd) {
       msg[kChunkedLength] += len;
       ret = msg[kChunkedLength] < msg[kHighWaterMark];
     } else {
-      msg._send(NumberPrototypeToString(len, 16), 'latin1', null);
+      msg._send(len.toString(16), 'latin1', null);
       msg._send(crlf_buf, null, null);
       msg._send(chunk, encoding, null, len);
       ret = msg._send(crlf_buf, null, callback);
@@ -1053,7 +1049,7 @@ OutgoingMessage.prototype.addTrailers = function addTrailers(headers) {
     const isArrayValue = ArrayIsArray(value);
     if (
       isArrayValue && value.length > 1 &&
-      (!this[kUniqueHeaders] || !this[kUniqueHeaders].has(StringPrototypeToLowerCase(field)))
+      (!this[kUniqueHeaders] || !this[kUniqueHeaders].has(field.toLowerCase()))
     ) {
       for (let j = 0, l = value.length; j < l; j++) {
         if (checkInvalidHeaderChar(value[j])) {
@@ -1064,7 +1060,7 @@ OutgoingMessage.prototype.addTrailers = function addTrailers(headers) {
       }
     } else {
       if (isArrayValue) {
-        value = ArrayPrototypeJoin(value, '; ');
+        value = value.join('; ');
       }
 
       if (checkInvalidHeaderChar(value)) {
