@@ -476,7 +476,7 @@ function createFromString(string, ops, length = ops.byteLength(string)) {
 
 function fromString(string, encoding) {
   let ops;
-  if (!encoding || encoding === 'utf8') {
+  if (!encoding || encoding === 'utf8' || typeof encoding !== 'string') {
     ops = encodingOps.utf8;
   } else {
     ops = getEncodingOps(encoding);
@@ -843,12 +843,12 @@ Buffer.prototype.toString = function toString(encoding, start, end) {
   else if (start >= len)
     return '';
   else
-    start |= 0;
+    start = MathTrunc(start) || 0;
 
   if (end === undefined || end > len)
     end = len;
   else
-    end |= 0;
+    end = MathTrunc(end) || 0;
 
   if (end <= start)
     return '';
