@@ -47,12 +47,11 @@ const {
   StringPrototypeSplit,
   Symbol,
   SymbolAsyncIterator,
-  SymbolDispose,
   SymbolFor,
 } = primordials;
 const kRejection = SymbolFor('nodejs.rejection');
 
-const { kEmptyObject } = require('internal/util');
+const { SymbolDispose, kEmptyObject } = require('internal/util');
 
 const {
   inspect,
