@@ -10,9 +10,9 @@ const {
   PromiseResolve,
   SafeFinalizationRegistry,
   SafeSet,
+  SafeWeakRef,
   Symbol,
   SymbolToStringTag,
-  WeakRef,
 } = primordials;
 
 const {
@@ -39,6 +39,10 @@ const {
     ERR_INVALID_THIS,
   },
 } = require('internal/errors');
+const {
+  converters,
+  createSequenceConverter,
+} = require('internal/webidl');
 
 const {
   validateAbortSignal,
@@ -216,7 +220,7 @@ class AbortSignal extends EventTarget {
     signal[kTimeout] = true;
     clearTimeoutRegistry.register(
       signal,
-      setWeakAbortSignalTimeout(new WeakRef(signal), delay));
+      setWeakAbortSignalTimeout(new SafeWeakRef(signal), delay));
     return signal;
   }
 
@@ -225,22 +229,26 @@ class AbortSignal extends EventTarget {
    * @returns {AbortSignal}
    */
   static any(signals) {
-    validateAbortSignalArray(signals, 'signals');
+    const signalsArray = createSequenceConverter(
+      converters.any,
+    )(signals);
+
+    validateAbortSignalArray(signalsArray, 'signals');
     const resultSignal = new AbortSignal(kDontThrowSymbol, { composite: true });
-    if (!signals.length) {
+    if (!signalsArray.length) {
       return resultSignal;
     }
-    const resultSignalWeakRef = new WeakRef(resultSignal);
+    const resultSignalWeakRef = new SafeWeakRef(resultSignal);
     resultSignal[kSourceSignals] = new SafeSet();
-    for (let i = 0; i < signals.length; i++) {
-      const signal = signals[i];
+    for (let i = 0; i < signalsArray.length; i++) {
+      const signal = signalsArray[i];
       if (signal.aborted) {
         abortSignal(resultSignal, signal.reason);
         return resultSignal;
       }
       signal[kDependantSignals] ??= new SafeSet();
       if (!signal[kComposite]) {
-        resultSignal[kSourceSignals].add(new WeakRef(signal));
+        resultSignal[kSourceSignals].add(new SafeWeakRef(signal));
         signal[kDependantSignals].add(resultSignalWeakRef);
       } else if (!signal[kSourceSignals]) {
         continue;
