@@ -39,7 +39,6 @@ const {
   codes: {
     ERR_CONSOLE_WRITABLE_STREAM,
     ERR_INCOMPATIBLE_OPTION_PAIR,
-    ERR_INVALID_ARG_VALUE,
   },
   isStackOverflowError,
 } = require('internal/errors');
@@ -47,6 +46,7 @@ const {
   validateArray,
   validateInteger,
   validateObject,
+  validateOneOf,
 } = require('internal/validators');
 const { previewEntries } = internalBinding('util');
 const { Buffer: { isBuffer } } = require('buffer');
@@ -78,7 +78,6 @@ function lazyUtilColors() {
 }
 
 // Track amount of indentation required via `console.group()`.
-const kGroupIndent = Symbol('kGroupIndent');
 const kGroupIndentationWidth = Symbol('kGroupIndentWidth');
 const kFormatForStderr = Symbol('kFormatForStderr');
 const kFormatForStdout = Symbol('kFormatForStdout');
@@ -91,7 +90,6 @@ const kBindStreamsEager = Symbol('kBindStreamsEager');
 const kBindStreamsLazy = Symbol('kBindStreamsLazy');
 const kUseStdout = Symbol('kUseStdout');
 const kUseStderr = Symbol('kUseStderr');
-const kInternalTimeLogImpl = Symbol('kInternalTimeLogImpl');
 
 const optionsMap = new SafeWeakMap();
 function Console(options /* or: stdout, stderr, ignoreErrors = true */) {
@@ -126,8 +124,7 @@ function Console(options /* or: stdout, stderr, ignoreErrors = true */) {
     throw new ERR_CONSOLE_WRITABLE_STREAM('stderr');
   }
 
-  if (typeof colorMode !== 'boolean' && colorMode !== 'auto')
-    throw new ERR_INVALID_ARG_VALUE('colorMode', colorMode);
+  validateOneOf(colorMode, 'colorMode', ['auto', true, false]);
 
   if (groupIndentation !== undefined) {
     validateInteger(groupIndentation, 'groupIndentation',
@@ -178,6 +175,8 @@ ObjectDefineProperty(Console, SymbolHasInstance, {
 const kColorInspectOptions = { colors: true };
 const kNoColorInspectOptions = {};
 
+const internalIndentationMap = new SafeWeakMap();
+
 ObjectDefineProperties(Console.prototype, {
   [kBindStreamsEager]: {
     __proto__: null,
@@ -247,7 +246,6 @@ ObjectDefineProperties(Console.prototype, {
         [kCounts]: { __proto__: null, ...consolePropAttributes, value: new SafeMap() },
         [kColorMode]: { __proto__: null, ...consolePropAttributes, value: colorMode },
         [kIsConsole]: { __proto__: null, ...consolePropAttributes, value: true },
-        [kGroupIndent]: { __proto__: null, ...consolePropAttributes, value: '' },
         [kGroupIndentationWidth]: {
           __proto__: null,
           ...consolePropAttributes,
@@ -268,7 +266,7 @@ ObjectDefineProperties(Console.prototype, {
     ...consolePropAttributes,
     value: function(streamSymbol, string, color = '') {
       const ignoreErrors = this._ignoreErrors;
-      const groupIndent = this[kGroupIndent];
+      const groupIndent = internalIndentationMap.get(this) || '';
 
       const useStdout = streamSymbol === kUseStdout;
       const stream = useStdout ? this._stdout : this._stderr;
@@ -372,11 +370,11 @@ function createWriteErrorHandler(instance, streamSymbol) {
   };
 }
 
-function timeLogImpl(label, formatted, args) {
+function timeLogImpl(consoleRef, label, formatted, args) {
   if (args === undefined) {
-    this.log('%s: %s', label, formatted);
+    consoleRef.log('%s: %s', label, formatted);
   } else {
-    this.log('%s: %s', label, formatted, ...new SafeArrayIterator(args));
+    consoleRef.log('%s: %s', label, formatted, ...new SafeArrayIterator(args));
   }
 }
 
@@ -407,17 +405,11 @@ const consoleMethods = {
   },
 
   timeEnd(label = 'default') {
-    if (this[kInternalTimeLogImpl] === undefined)
-      this[kInternalTimeLogImpl] = FunctionPrototypeBind(timeLogImpl, this);
-
-    timeEnd(this._times, kTraceConsoleCategory, 'console.timeEnd()', kNone, this[kInternalTimeLogImpl], label, `time::${label}`);
+    timeEnd(this._times, kTraceConsoleCategory, 'console.timeEnd()', kNone, (label, formatted, args) => timeLogImpl(this, label, formatted, args), label, `time::${label}`);
   },
 
   timeLog(label = 'default', ...data) {
-    if (this[kInternalTimeLogImpl] === undefined)
-      this[kInternalTimeLogImpl] = FunctionPrototypeBind(timeLogImpl, this);
-
-    timeLog(this._times, kTraceConsoleCategory, 'console.timeLog()', kNone, this[kInternalTimeLogImpl], label, `time::${label}`, data);
+    timeLog(this._times, kTraceConsoleCategory, 'console.timeLog()', kNone, (label, formatted, args) => timeLogImpl(this, label, formatted, args), label, `time::${label}`, data);
   },
 
   trace: function trace(...args) {
@@ -489,16 +481,22 @@ const consoleMethods = {
     if (data.length > 0) {
       ReflectApply(this.log, this, data);
     }
-    this[kGroupIndent] +=
-      StringPrototypeRepeat(' ', this[kGroupIndentationWidth]);
+
+    let currentIndentation = internalIndentationMap.get(this) || '';
+    currentIndentation += StringPrototypeRepeat(' ', this[kGroupIndentationWidth]);
+
+    internalIndentationMap.set(this, currentIndentation);
   },
 
   groupEnd() {
-    this[kGroupIndent] = StringPrototypeSlice(
-      this[kGroupIndent],
+    const currentIndentation = internalIndentationMap.get(this) || '';
+    const newIndentation = StringPrototypeSlice(
+      currentIndentation,
       0,
-      this[kGroupIndent].length - this[kGroupIndentationWidth],
+      currentIndentation.length - this[kGroupIndentationWidth],
     );
+
+    internalIndentationMap.set(this, newIndentation);
   },
 
   // https://console.spec.whatwg.org/#table
