@@ -49,6 +49,7 @@ const {
   kEmptyObject,
   once,
   deprecate,
+  isWindows,
 } = require('internal/util');
 const { toPathIfFileURL } = require('internal/url');
 const {
@@ -145,8 +146,6 @@ const kWriteFileMaxChunkSize = 512 * 1024;
 
 const kMaxUserId = 2 ** 32 - 1;
 
-const isWindows = process.platform === 'win32';
-
 let fs;
 function lazyLoadFs() {
   if (!fs) {
