@@ -29,20 +29,22 @@ const {
   ERR_UNKNOWN_MODULE_FORMAT,
 } = require('internal/errors').codes;
 const { getOptionValue } = require('internal/options');
-const { isURL, pathToFileURL, URL } = require('internal/url');
-const { emitExperimentalWarning, kEmptyObject } = require('internal/util');
+const { isURL, pathToFileURL, URLParse } = require('internal/url');
+const { kEmptyObject } = require('internal/util');
 const {
   compileSourceTextModule,
   getDefaultConditions,
 } = require('internal/modules/esm/utils');
 const { kImplicitTypeAttribute } = require('internal/modules/esm/assert');
-const { canParse } = internalBinding('url');
 const { ModuleWrap, kEvaluating, kEvaluated } = internalBinding('module_wrap');
 const {
   urlToFilename,
 } = require('internal/modules/helpers');
 let defaultResolve, defaultLoad, defaultLoadSync, importMetaInitializer;
 
+const { tracingChannel } = require('diagnostics_channel');
+const onImport = tracingChannel('module.import');
+
 /**
  * @typedef {import('url').URL} URL
  */
@@ -143,9 +145,6 @@ class ModuleLoader {
   #customizations;
 
   constructor(customizations) {
-    if (getOptionValue('--experimental-network-imports')) {
-      emitExperimentalWarning('Network Imports');
-    }
     this.setCustomizations(customizations);
   }
 
@@ -211,10 +210,17 @@ class ModuleLoader {
       return compileSourceTextModule(url, source, this);
     };
     const { ModuleJob } = require('internal/modules/esm/module_job');
-    const job = new ModuleJob(
-      this, url, undefined, evalInstance, false, false);
-    this.loadCache.set(url, undefined, job);
-    const { module } = await job.run(isEntryPoint);
+    const module = await onImport.tracePromise(async () => {
+      const job = new ModuleJob(
+        this, url, undefined, evalInstance, false, false);
+      this.loadCache.set(url, undefined, job);
+      const { module } = await job.run(isEntryPoint);
+      return module;
+    }, {
+      __proto__: null,
+      parentURL: '<eval>',
+      url,
+    });
 
     return {
       __proto__: null,
@@ -271,7 +277,7 @@ class ModuleLoader {
    * @param {string} source Source code. TODO(joyeecheung): pass the raw buffer.
    * @param {string} isMain Whether this module is a main module.
    * @param {CJSModule|undefined} parent Parent module, if any.
-   * @returns {{ModuleWrap}}
+   * @returns {{wrap: ModuleWrap, namespace: ModuleNamespaceObject}}
    */
   importSyncForRequire(mod, filename, source, isMain, parent) {
     const url = pathToFileURL(filename).href;
@@ -296,7 +302,7 @@ class ModuleLoader {
         }
         throw new ERR_REQUIRE_CYCLE_MODULE(message);
       }
-      return job.module.getNamespaceSync();
+      return { wrap: job.module, namespace: job.module.getNamespaceSync() };
     }
     // TODO(joyeecheung): refactor this so that we pre-parse in C++ and hit the
     // cache here, or use a carrier object to carry the compiled module script
@@ -308,7 +314,7 @@ class ModuleLoader {
     job = new ModuleJobSync(this, url, kEmptyObject, wrap, isMain, inspectBrk);
     this.loadCache.set(url, kImplicitTypeAttribute, job);
     mod[kRequiredModuleSymbol] = job.module;
-    return job.runSync().namespace;
+    return { wrap: job.module, namespace: job.runSync().namespace };
   }
 
   /**
@@ -320,10 +326,9 @@ class ModuleLoader {
    * @returns {ModuleJobBase}
    */
   getModuleJobForRequire(specifier, parentURL, importAttributes) {
-    assert(getOptionValue('--experimental-require-module'));
-
-    if (canParse(specifier)) {
-      const protocol = new URL(specifier).protocol;
+    const parsed = URLParse(specifier);
+    if (parsed != null) {
+      const protocol = parsed.protocol;
       if (protocol === 'https:' || protocol === 'http:') {
         throw new ERR_NETWORK_IMPORT_DISALLOWED(specifier, parentURL,
                                                 'ES modules cannot be loaded by require() from the network');
@@ -339,6 +344,9 @@ class ModuleLoader {
     }
 
     const { url, format } = resolveResult;
+    if (!getOptionValue('--experimental-require-module')) {
+      throw new ERR_REQUIRE_ESM(url, true);
+    }
     const resolvedImportAttributes = resolveResult.importAttributes ?? importAttributes;
     let job = this.loadCache.get(url, resolvedImportAttributes.type);
     if (job !== undefined) {
@@ -470,15 +478,21 @@ class ModuleLoader {
    * @returns {Promise<ModuleExports>}
    */
   async import(specifier, parentURL, importAttributes, isEntryPoint = false) {
-    const moduleJob = await this.getModuleJob(specifier, parentURL, importAttributes);
-    const { module } = await moduleJob.run(isEntryPoint);
-    return module.getNamespace();
+    return onImport.tracePromise(async () => {
+      const moduleJob = await this.getModuleJob(specifier, parentURL, importAttributes);
+      const { module } = await moduleJob.run(isEntryPoint);
+      return module.getNamespace();
+    }, {
+      __proto__: null,
+      parentURL,
+      url: specifier,
+    });
   }
 
   /**
    * @see {@link CustomizedModuleLoader.register}
    */
-  register(specifier, parentURL, data, transferList) {
+  register(specifier, parentURL, data, transferList, isInternal) {
     if (!this.#customizations) {
       // `CustomizedModuleLoader` is defined at the bottom of this file and
       // available well before this line is ever invoked. This is here in
@@ -486,7 +500,7 @@ class ModuleLoader {
       // eslint-disable-next-line no-use-before-define
       this.setCustomizations(new CustomizedModuleLoader());
     }
-    return this.#customizations.register(`${specifier}`, `${parentURL}`, data, transferList);
+    return this.#customizations.register(`${specifier}`, `${parentURL}`, data, transferList, isInternal);
   }
 
   /**
@@ -499,6 +513,7 @@ class ModuleLoader {
    * @returns {{ format: string, url: URL['href'] }}
    */
   resolve(originalSpecifier, parentURL, importAttributes) {
+    originalSpecifier = `${originalSpecifier}`;
     if (this.#customizations) {
       return this.#customizations.resolve(originalSpecifier, parentURL, importAttributes);
     }
@@ -517,6 +532,7 @@ class ModuleLoader {
    * `import.meta.resolve` which must happen synchronously.
    */
   resolveSync(originalSpecifier, parentURL, importAttributes) {
+    originalSpecifier = `${originalSpecifier}`;
     if (this.#customizations) {
       return this.#customizations.resolveSync(originalSpecifier, parentURL, importAttributes);
     }
@@ -563,13 +579,18 @@ class ModuleLoader {
       this.#customizations.loadSync(url, context) :
       defaultLoadSync(url, context);
     let format = result?.format;
-    if (format === 'module') {
+    if (format === 'module' || format === 'module-typescript') {
       throw new ERR_REQUIRE_ESM(url, true);
     }
     if (format === 'commonjs') {
       format = 'require-commonjs';
       result = { __proto__: result, format };
     }
+    if (format === 'commonjs-typescript') {
+      format = 'require-commonjs-typescript';
+      result = { __proto__: result, format };
+    }
+
     this.validateLoadResult(url, format);
     return result;
   }
@@ -618,10 +639,11 @@ class CustomizedModuleLoader {
    * @param {any} [data] Arbitrary data to be passed from the custom loader
    * (user-land) to the worker.
    * @param {any[]} [transferList] Objects in `data` that are changing ownership
+   * @param {boolean} [isInternal] For internal loaders that should not be publicly exposed.
    * @returns {{ format: string, url: URL['href'] }}
    */
-  register(originalSpecifier, parentURL, data, transferList) {
-    return hooksProxy.makeSyncRequest('register', transferList, originalSpecifier, parentURL, data);
+  register(originalSpecifier, parentURL, data, transferList, isInternal) {
+    return hooksProxy.makeSyncRequest('register', transferList, originalSpecifier, parentURL, data, isInternal);
   }
 
   /**
