@@ -16,15 +16,9 @@ const {
   String,
   StringPrototypeStartsWith,
   Symbol,
-  SymbolAsyncDispose,
-  SymbolDispose,
   globalThis,
 } = primordials;
 
-const {
-  Intl,
-} = globalThis;
-
 const {
   getOptionValue,
   refreshOptions,
@@ -37,6 +31,8 @@ const {
   defineReplaceableLazyAttribute,
   setupCoverageHooks,
   emitExperimentalWarning,
+  SymbolAsyncDispose,
+  SymbolDispose,
 } = require('internal/util');
 
 const {
@@ -347,7 +343,6 @@ function setupNavigator() {
 
   // https://html.spec.whatwg.org/multipage/system-state.html#the-navigator-object
   exposeLazyInterfaces(globalThis, 'internal/navigator', ['Navigator']);
-  internalBinding('config').language = Intl?.Collator().resolvedOptions().locale || 'en-US';
   defineReplaceableLazyAttribute(globalThis, 'internal/navigator', ['navigator'], false);
 }
 
