@@ -30,7 +30,6 @@ const {
   StringPrototypeTrim,
   Symbol,
   SymbolAsyncIterator,
-  SymbolDispose,
 } = primordials;
 
 const { codes: {
@@ -45,7 +44,7 @@ const {
   validateString,
   validateUint32,
 } = require('internal/validators');
-const { kEmptyObject } = require('internal/util');
+const { SymbolDispose, kEmptyObject } = require('internal/util');
 const {
   inspect,
   getStringWidth,
