@@ -35,7 +35,6 @@ const {
   Promise,
   StringPrototypeToLowerCase,
   Symbol,
-  SymbolAsyncDispose,
   SymbolHasInstance,
 } = primordials;
 
@@ -47,6 +46,9 @@ const Stream = require('internal/streams/legacy').Stream;
 const { Buffer } = require('buffer');
 const destroyImpl = require('internal/streams/destroy');
 const eos = require('internal/streams/end-of-stream');
+const {
+  SymbolAsyncDispose,
+} = require('internal/util');
 
 const {
   addAbortSignal,
@@ -300,7 +302,7 @@ ObjectDefineProperties(WritableState.prototype, {
 });
 
 function WritableState(options, stream, isDuplex) {
-  // Bit map field to store WritableState more effciently with 1 bit per field
+  // Bit map field to store WritableState more efficiently with 1 bit per field
   // instead of a V8 slot per field.
   this[kState] = kSync | kConstructed | kEmitClose | kAutoDestroy;
 
