@@ -23,6 +23,7 @@ const {
   mkdtempSync,
   opendirSync,
   readFileSync,
+  rmSync,
 } = require('fs');
 const { setupCoverageHooks } = require('internal/util');
 const { tmpdir } = require('os');
@@ -50,12 +51,13 @@ class CoverageLine {
 }
 
 class TestCoverage {
-  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory, excludeGlobs, includeGlobs) {
+  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory, excludeGlobs, includeGlobs, thresholds) {
     this.coverageDirectory = coverageDirectory;
     this.originalCoverageDirectory = originalCoverageDirectory;
     this.workingDirectory = workingDirectory;
     this.excludeGlobs = excludeGlobs;
     this.includeGlobs = includeGlobs;
+    this.thresholds = thresholds;
   }
 
   #sourceLines = new SafeMap();
@@ -142,6 +144,7 @@ class TestCoverage {
         coveredBranchPercent: 0,
         coveredFunctionPercent: 0,
       },
+      thresholds: this.thresholds,
     };
 
     if (!coverage) {
@@ -272,27 +275,34 @@ class TestCoverage {
   cleanup() {
     // Restore the original value of process.env.NODE_V8_COVERAGE. Then, copy
     // all of the created coverage files to the original coverage directory.
+    internalBinding('profiler').endCoverage();
+
     if (this.originalCoverageDirectory === undefined) {
       delete process.env.NODE_V8_COVERAGE;
-      return;
+    } else {
+      process.env.NODE_V8_COVERAGE = this.originalCoverageDirectory;
+      let dir;
+
+      try {
+        mkdirSync(this.originalCoverageDirectory, { __proto__: null, recursive: true });
+        dir = opendirSync(this.coverageDirectory);
+
+        for (let entry; (entry = dir.readSync()) !== null;) {
+          const src = join(this.coverageDirectory, entry.name);
+          const dst = join(this.originalCoverageDirectory, entry.name);
+          copyFileSync(src, dst);
+        }
+      } finally {
+        if (dir) {
+          dir.closeSync();
+        }
+      }
     }
 
-    process.env.NODE_V8_COVERAGE = this.originalCoverageDirectory;
-    let dir;
-
     try {
-      mkdirSync(this.originalCoverageDirectory, { __proto__: null, recursive: true });
-      dir = opendirSync(this.coverageDirectory);
-
-      for (let entry; (entry = dir.readSync()) !== null;) {
-        const src = join(this.coverageDirectory, entry.name);
-        const dst = join(this.originalCoverageDirectory, entry.name);
-        copyFileSync(src, dst);
-      }
-    } finally {
-      if (dir) {
-        dir.closeSync();
-      }
+      rmSync(this.coverageDirectory, { __proto__: null, recursive: true });
+    } catch {
+      // Ignore cleanup errors.
     }
   }
 
@@ -340,8 +350,8 @@ class TestCoverage {
       const { data, lineLengths } = sourceMapCache[url];
       let offset = 0;
       const executedLines = ArrayPrototypeMap(lineLengths, (length, i) => {
-        const coverageLine = new CoverageLine(i + 1, offset, null, length);
-        offset += length;
+        const coverageLine = new CoverageLine(i + 1, offset, null, length + 1);
+        offset += length + 1;
         return coverageLine;
       });
       if (data.sourcesContent != null) {
@@ -497,6 +507,12 @@ function setupCoverage(options) {
     cwd,
     options.coverageExcludeGlobs,
     options.coverageIncludeGlobs,
+    {
+      __proto__: null,
+      line: options.lineCoverage,
+      branch: options.branchCoverage,
+      function: options.functionCoverage,
+    },
   );
 }
 
