@@ -12,10 +12,10 @@ module.exports = async function* dot(source) {
   const failedTests = [];
   for await (const { type, data } of source) {
     if (type === 'test:pass') {
-      yield '.';
+      yield `${colors.green}.${colors.clear}`;
     }
     if (type === 'test:fail') {
-      yield 'X';
+      yield `${colors.red}X${colors.clear}`;
       ArrayPrototypePush(failedTests, data);
     }
     if ((type === 'test:fail' || type === 'test:pass') && ++count === columns) {
