@@ -11,6 +11,7 @@ const {
   FunctionPrototype,
   MathMax,
   Number,
+  NumberPrototypeToFixed,
   ObjectDefineProperty,
   ObjectSeal,
   PromisePrototypeThen,
@@ -25,9 +26,9 @@ const {
   StringPrototypeStartsWith,
   StringPrototypeTrim,
   Symbol,
-  SymbolDispose,
 } = primordials;
 const { getCallerLocation } = internalBinding('util');
+const { exitCodes: { kGenericUserError } } = internalBinding('errors');
 const { addAbortListener } = require('internal/events/abort_listener');
 const { queueMicrotask } = require('internal/process/task_queues');
 const { AsyncResource } = require('async_hooks');
@@ -50,6 +51,7 @@ const {
   createDeferredPromise,
   kEmptyObject,
   once: runOnce,
+  SymbolDispose,
 } = require('internal/util');
 const { isPromise } = require('internal/util/types');
 const {
@@ -62,6 +64,7 @@ const { setTimeout } = require('timers');
 const { TIMEOUT_MAX } = require('internal/timers');
 const { fileURLToPath } = require('internal/url');
 const { availableParallelism } = require('os');
+const { innerOk } = require('internal/assert/utils');
 const { bigint: hrtime } = process.hrtime;
 const kCallbackAndPromisePresent = 'callbackAndPromisePresent';
 const kCancelledByParent = 'cancelledByParent';
@@ -111,7 +114,6 @@ function lazyAssertObject(harness) {
       'notDeepStrictEqual',
       'notEqual',
       'notStrictEqual',
-      'ok',
       'rejects',
       'strictEqual',
       'throws',
@@ -261,6 +263,16 @@ class TestContext {
           return ReflectApply(method, this, args);
         };
       });
+
+      // This is a hack. It allows the innerOk function to collect the stacktrace from the correct starting point.
+      function ok(...args) {
+        if (plan !== null) {
+          plan.actual++;
+        }
+        innerOk(ok, args.length, ...args);
+      }
+
+      assert.ok = ok;
     }
     return this.#assert;
   }
@@ -393,8 +405,10 @@ class Test extends AsyncResource {
     this.parent = parent;
     this.testNumber = 0;
     this.outputSubtestCount = 0;
-    this.filteredSubtestCount = 0;
+    this.diagnostics = [];
     this.filtered = false;
+    this.filteredByName = false;
+    this.hasOnlyTests = false;
 
     if (parent === null) {
       this.root = this;
@@ -408,45 +422,51 @@ class Test extends AsyncResource {
       this.childNumber = 0;
       this.timeout = kDefaultTimeout;
       this.entryFile = entryFile;
-      this.hooks = {
-        __proto__: null,
-        before: [],
-        after: [],
-        beforeEach: [],
-        afterEach: [],
-        ownAfterEachCount: 0,
-      };
     } else {
       const nesting = parent.parent === null ? parent.nesting :
         parent.nesting + 1;
+      const { config, isFilteringByName, isFilteringByOnly } = parent.root.harness;
 
       this.root = parent.root;
       this.harness = null;
-      this.config = this.root.harness.config;
+      this.config = config;
       this.concurrency = parent.concurrency;
       this.nesting = nesting;
-      this.only = only ?? (parent.only && !parent.runOnlySubtests);
+      this.only = only;
       this.reporter = parent.reporter;
       this.runOnlySubtests = false;
       this.childNumber = parent.subtests.length + 1;
       this.timeout = parent.timeout;
       this.entryFile = parent.entryFile;
-      this.hooks = {
-        __proto__: null,
-        before: [],
-        after: [],
-        beforeEach: ArrayPrototypeSlice(parent.hooks.beforeEach),
-        afterEach: ArrayPrototypeSlice(parent.hooks.afterEach),
-        ownAfterEachCount: 0,
-      };
 
-      if (this.willBeFiltered()) {
-        this.filtered = true;
-        this.parent.filteredSubtestCount++;
+      if (isFilteringByName) {
+        this.filteredByName = this.willBeFilteredByName();
+        if (!this.filteredByName) {
+          for (let t = this.parent; t !== null && t.filteredByName; t = t.parent) {
+            t.filteredByName = false;
+          }
+        }
       }
 
-      if (this.config.only && only === false) {
-        fn = noop;
+      if (isFilteringByOnly) {
+        if (this.only) {
+          // If filtering impacts the tests within a suite, then the suite only
+          // runs those tests. If filtering does not impact the tests within a
+          // suite, then all tests are run.
+          this.parent.runOnlySubtests = true;
+
+          if (this.parent === this.root || this.parent.startTime === null) {
+            for (let t = this.parent; t !== null && !t.hasOnlyTests; t = t.parent) {
+              t.hasOnlyTests = true;
+            }
+          }
+        } else if (this.only === false) {
+          fn = noop;
+        }
+      } else if (only || this.parent.runOnlySubtests) {
+        const warning =
+          "'only' and 'runOnly' require the --test-only command-line option.";
+        this.diagnostic(warning);
       }
     }
 
@@ -505,7 +525,6 @@ class Test extends AsyncResource {
     this.endTime = null;
     this.passed = false;
     this.error = null;
-    this.diagnostics = [];
     this.message = typeof skip === 'string' ? skip :
       typeof todo === 'string' ? todo : null;
     this.activeSubtests = 0;
@@ -514,12 +533,14 @@ class Test extends AsyncResource {
     this.subtests = [];
     this.waitingOn = 0;
     this.finished = false;
-
-    if (!this.config.only && (only || this.parent?.runOnlySubtests)) {
-      const warning =
-        "'only' and 'runOnly' require the --test-only command-line option.";
-      this.diagnostic(warning);
-    }
+    this.hooks = {
+      __proto__: null,
+      before: [],
+      after: [],
+      beforeEach: [],
+      afterEach: [],
+      ownAfterEachCount: 0,
+    };
 
     if (loc === undefined) {
       this.loc = undefined;
@@ -548,9 +569,27 @@ class Test extends AsyncResource {
     }
   }
 
-  willBeFiltered() {
-    if (this.config.only && !this.only) return true;
+  applyFilters() {
+    if (this.error) {
+      // Never filter out errors.
+      return;
+    }
 
+    if (this.filteredByName) {
+      this.filtered = true;
+      return;
+    }
+
+    if (this.root.harness.isFilteringByOnly && !this.only && !this.hasOnlyTests) {
+      if (this.parent.runOnlySubtests ||
+          this.parent.hasOnlyTests ||
+          this.only === false) {
+        this.filtered = true;
+      }
+    }
+  }
+
+  willBeFilteredByName() {
     const { testNamePatterns, testSkipPatterns } = this.config;
 
     if (testNamePatterns && !testMatchesPattern(this, testNamePatterns)) {
@@ -691,6 +730,21 @@ class Test extends AsyncResource {
     this.abortController.abort();
   }
 
+  computeInheritedHooks() {
+    if (this.parent.hooks.beforeEach.length > 0) {
+      ArrayPrototypeUnshift(
+        this.hooks.beforeEach,
+        ...ArrayPrototypeSlice(this.parent.hooks.beforeEach),
+      );
+    }
+
+    if (this.parent.hooks.afterEach.length > 0) {
+      ArrayPrototypePushApply(
+        this.hooks.afterEach, ArrayPrototypeSlice(this.parent.hooks.afterEach),
+      );
+    }
+  }
+
   createHook(name, fn, options) {
     validateOneOf(name, 'hook name', kHookNames);
     // eslint-disable-next-line no-use-before-define
@@ -715,7 +769,6 @@ class Test extends AsyncResource {
     } else {
       ArrayPrototypePush(this.hooks[name], hook);
     }
-    return hook;
   }
 
   fail(err) {
@@ -749,12 +802,10 @@ class Test extends AsyncResource {
     ArrayPrototypePush(this.diagnostics, message);
   }
 
-  get shouldFilter() {
-    return this.filtered && this.parent?.filteredSubtestCount > 0;
-  }
-
   start() {
-    if (this.shouldFilter) {
+    this.applyFilters();
+
+    if (this.filtered) {
       noopTestStream ??= new TestsStream();
       this.reporter = noopTestStream;
       this.run = this.filteredRun;
@@ -766,7 +817,7 @@ class Test extends AsyncResource {
     // it. Otherwise, return a Promise to the caller and mark the test as
     // pending for later execution.
     this.reporter.enqueue(this.nesting, this.loc, this.name);
-    if (!this.root.harness.allowTestsToRun || !this.parent.hasConcurrency()) {
+    if (this.root.harness.buildPromise || !this.parent.hasConcurrency()) {
       const deferred = createDeferredPromise();
 
       deferred.test = this;
@@ -817,6 +868,7 @@ class Test extends AsyncResource {
   async run() {
     if (this.parent !== null) {
       this.parent.activeSubtests++;
+      this.computeInheritedHooks();
     }
     this.startTime ??= hrtime();
 
@@ -961,7 +1013,7 @@ class Test extends AsyncResource {
     this.mock?.reset();
 
     if (this.parent !== null) {
-      if (!this.shouldFilter) {
+      if (!this.filtered) {
         const report = this.getReportDetails();
         report.details.passed = this.passed;
         this.testNumber ||= ++this.parent.outputSubtestCount;
@@ -1001,6 +1053,25 @@ class Test extends AsyncResource {
       reporter.diagnostic(nesting, loc, `duration_ms ${this.duration()}`);
 
       if (coverage) {
+        const coverages = [
+          { __proto__: null, actual: coverage.totals.coveredLinePercent,
+            threshold: this.config.lineCoverage, name: 'line' },
+
+          { __proto__: null, actual: coverage.totals.coveredBranchPercent,
+            threshold: this.config.branchCoverage, name: 'branch' },
+
+          { __proto__: null, actual: coverage.totals.coveredFunctionPercent,
+            threshold: this.config.functionCoverage, name: 'function' },
+        ];
+
+        for (let i = 0; i < coverages.length; i++) {
+          const { threshold, actual, name } = coverages[i];
+          if (actual < threshold) {
+            process.exitCode = kGenericUserError;
+            reporter.diagnostic(nesting, loc, `Error: ${NumberPrototypeToFixed(actual, 2)}% ${name} coverage does not meet threshold of ${threshold}%.`);
+          }
+        }
+
         reporter.coverage(nesting, loc, coverage);
       }
 
@@ -1131,7 +1202,7 @@ class TestHook extends Test {
   getRunArgs() {
     return this.#args;
   }
-  willBeFiltered() {
+  willBeFilteredByName() {
     return false;
   }
   postRun() {
@@ -1164,7 +1235,6 @@ class Suite extends Test {
       this.fn = options.fn || this.fn;
       this.skipped = false;
     }
-    this.runOnlySubtests = this.config.only;
 
     try {
       const { ctx, args } = this.getRunArgs();
@@ -1188,21 +1258,6 @@ class Suite extends Test {
 
   postBuild() {
     this.buildPhaseFinished = true;
-    if (this.filtered &&
-        (this.filteredSubtestCount !== this.subtests.length || this.error)) {
-      // A suite can transition from filtered to unfiltered based on the
-      // tests that it contains - in case of children matching patterns.
-      this.filtered = false;
-      this.parent.filteredSubtestCount--;
-    } else if (
-      this.config.only &&
-      this.config.testNamePatterns == null &&
-      this.config.testSkipPatterns == null &&
-      this.filteredSubtestCount === this.subtests.length
-    ) {
-      // If no subtests are marked as "only", run them all
-      this.filteredSubtestCount = 0;
-    }
   }
 
   getRunArgs() {
@@ -1211,6 +1266,7 @@ class Suite extends Test {
   }
 
   async run() {
+    this.computeInheritedHooks();
     const hookArgs = this.getRunArgs();
 
     let stopPromise;
