@@ -39,6 +39,7 @@ const {
   kIsNodeError,
 } = require('internal/errors');
 const { compose } = require('stream');
+const { validateInteger } = require('internal/validators');
 
 const coverageColors = {
   __proto__: null,
@@ -50,7 +51,7 @@ const coverageColors = {
 const kMultipleCallbackInvocations = 'multipleCallbackInvocations';
 const kRegExpPattern = /^\/(.*)\/([a-z]*)$/;
 
-const kPatterns = ['test', 'test/**/*', 'test-*', '*[.-_]test'];
+const kPatterns = ['test', 'test/**/*', 'test-*', '*[._-]test'];
 const kDefaultPattern = `**/{${ArrayPrototypeJoin(kPatterns, ',')}}.?(c|m)js`;
 
 
@@ -194,12 +195,16 @@ function parseCommandLine() {
   let concurrency;
   let coverageExcludeGlobs;
   let coverageIncludeGlobs;
+  let lineCoverage;
+  let branchCoverage;
+  let functionCoverage;
   let destinations;
-  let only;
+  let isolation;
+  let only = getOptionValue('--test-only');
   let reporters;
   let shard;
-  let testNamePatterns;
-  let testSkipPatterns;
+  let testNamePatterns = mapPatternFlagToRegExArray('--test-name-pattern');
+  let testSkipPatterns = mapPatternFlagToRegExArray('--test-skip-pattern');
   let timeout;
 
   if (isChildProcessV8) {
@@ -230,10 +235,17 @@ function parseCommandLine() {
   }
 
   if (isTestRunner) {
+    isolation = getOptionValue('--experimental-test-isolation');
     timeout = getOptionValue('--test-timeout') || Infinity;
-    concurrency = getOptionValue('--test-concurrency') || true;
-    only = false;
-    testNamePatterns = null;
+
+    if (isolation === 'none') {
+      concurrency = 1;
+    } else {
+      concurrency = getOptionValue('--test-concurrency') || true;
+      only = false;
+      testNamePatterns = null;
+      testSkipPatterns = null;
+    }
 
     const shardOption = getOptionValue('--test-shard');
     if (shardOption) {
@@ -270,6 +282,14 @@ function parseCommandLine() {
   if (coverage) {
     coverageExcludeGlobs = getOptionValue('--test-coverage-exclude');
     coverageIncludeGlobs = getOptionValue('--test-coverage-include');
+
+    branchCoverage = getOptionValue('--test-coverage-branches');
+    lineCoverage = getOptionValue('--test-coverage-lines');
+    functionCoverage = getOptionValue('--test-coverage-functions');
+
+    validateInteger(branchCoverage, '--test-coverage-branches', 0, 100);
+    validateInteger(lineCoverage, '--test-coverage-lines', 0, 100);
+    validateInteger(functionCoverage, '--test-coverage-functions', 0, 100);
   }
 
   const setup = reporterScope.bind(async (rootReporter) => {
@@ -290,6 +310,10 @@ function parseCommandLine() {
     coverageIncludeGlobs,
     destinations,
     forceExit,
+    isolation,
+    branchCoverage,
+    functionCoverage,
+    lineCoverage,
     only,
     reporters,
     setup,
@@ -305,6 +329,16 @@ function parseCommandLine() {
   return globalTestOptions;
 }
 
+function mapPatternFlagToRegExArray(flagName) {
+  const patterns = getOptionValue(flagName);
+
+  if (patterns?.length > 0) {
+    return ArrayPrototypeMap(patterns, (re) => convertStringToRegExp(re, flagName));
+  }
+
+  return null;
+}
+
 function countCompletedTest(test, harness = test.root.harness) {
   if (test.nesting === 0) {
     harness.counters.topLevel++;
