@@ -1,7 +1,6 @@
 'use strict';
 
 const {
-  ArrayBufferPrototypeGetByteLength,
   ArrayFrom,
   ArrayIsArray,
   ArrayPrototypePush,
@@ -57,7 +56,6 @@ const {
 } = require('internal/errors');
 const { signals } = internalBinding('constants').os;
 const {
-  isArrayBufferDetached: _isArrayBufferDetached,
   guessHandleType: _guessHandleType,
   privateSymbols: {
     arrow_message_private_symbol,
@@ -798,15 +796,6 @@ function SideEffectFreeRegExpPrototypeSymbolSplit(regex, string, limit = undefin
   return getCrossRelmRegex(regex)[SymbolSplit](string, limit);
 }
 
-
-function isArrayBufferDetached(value) {
-  if (ArrayBufferPrototypeGetByteLength(value) === 0) {
-    return _isArrayBufferDetached(value);
-  }
-
-  return false;
-}
-
 /**
  * Helper function to lazy-load an initialize-once value.
  * @template T Return value of initializer
@@ -917,7 +906,6 @@ module.exports = {
   getSystemErrorMap,
   getSystemErrorName,
   guessHandleType,
-  isArrayBufferDetached,
   isError,
   isInsideNodeModules,
   isMacOS,
@@ -936,6 +924,14 @@ module.exports = {
   setupCoverageHooks,
   removeColors,
 
+  // Define Symbol.dispose and Symbol.asyncDispose
+  // Until these are defined by the environment.
+  // TODO(MoLow): Remove this polyfill once Symbol.dispose and Symbol.asyncDispose are available in primordials.
+  // eslint-disable-next-line node-core/prefer-primordials
+  SymbolDispose: Symbol.dispose || SymbolFor('nodejs.dispose'),
+  // eslint-disable-next-line node-core/prefer-primordials
+  SymbolAsyncDispose: Symbol.asyncDispose || SymbolFor('nodejs.asyncDispose'),
+
   // Symbol used to customize promisify conversion
   customPromisifyArgs: kCustomPromisifyArgsSymbol,
 
