@@ -502,8 +502,9 @@ function setEquiv(a, b, strict, memo) {
     for (const val of b) {
       // Primitive values have already been handled above.
       if (typeof val === 'object' && val !== null) {
-        if (!setHasEqualElement(set, val, strict, memo))
+        if (!a.has(val) && !setHasEqualElement(set, val, strict, memo)) {
           return false;
+        }
       } else if (!strict &&
                  !a.has(val) &&
                  !setHasEqualElement(set, val, strict, memo)) {
