@@ -133,6 +133,7 @@ function pad(value) {
 const kNone = 1 << 0;
 const kSkipLog = 1 << 1;
 const kSkipTrace = 1 << 2;
+const kShouldSkipAll = kSkipLog | kSkipTrace;
 
 const kSecond = 1000;
 const kMinute = 60 * kSecond;
@@ -377,8 +378,6 @@ function debugWithTimer(set, cb) {
   let debugLogCategoryEnabled = false;
   let timerFlags = kNone;
 
-  const skipAll = kSkipLog | kSkipTrace;
-
   function ensureTimerFlagsAreUpdated() {
     timerFlags &= ~kSkipTrace;
 
@@ -393,7 +392,7 @@ function debugWithTimer(set, cb) {
   function internalStartTimer(logLabel, traceLabel) {
     ensureTimerFlagsAreUpdated();
 
-    if (timerFlags === skipAll) {
+    if ((timerFlags & kShouldSkipAll) === kShouldSkipAll) {
       return;
     }
 
@@ -413,7 +412,7 @@ function debugWithTimer(set, cb) {
   function internalEndTimer(logLabel, traceLabel) {
     ensureTimerFlagsAreUpdated();
 
-    if (timerFlags === skipAll) {
+    if ((timerFlags & kShouldSkipAll) === kShouldSkipAll) {
       return;
     }
 
@@ -434,7 +433,7 @@ function debugWithTimer(set, cb) {
   function internalLogTimer(logLabel, traceLabel, args) {
     ensureTimerFlagsAreUpdated();
 
-    if (timerFlags === skipAll) {
+    if ((timerFlags & kShouldSkipAll) === kShouldSkipAll) {
       return;
     }
 
@@ -477,7 +476,7 @@ function debugWithTimer(set, cb) {
   const startTimer = (logLabel, traceLabel) => {
     init();
 
-    if (timerFlags !== skipAll)
+    if ((timerFlags & kShouldSkipAll) !== kShouldSkipAll)
       internalStartTimer(logLabel, traceLabel);
   };
 
@@ -487,7 +486,7 @@ function debugWithTimer(set, cb) {
   const endTimer = (logLabel, traceLabel) => {
     init();
 
-    if (timerFlags !== skipAll)
+    if ((timerFlags & kShouldSkipAll) !== kShouldSkipAll)
       internalEndTimer(logLabel, traceLabel);
   };
 
@@ -497,7 +496,7 @@ function debugWithTimer(set, cb) {
   const logTimer = (logLabel, traceLabel, args) => {
     init();
 
-    if (timerFlags !== skipAll)
+    if ((timerFlags & kShouldSkipAll) !== kShouldSkipAll)
       internalLogTimer(logLabel, traceLabel, args);
   };
 
