@@ -211,7 +211,7 @@ function newWritableStreamFromStreamWritable(streamWritable) {
   return new WritableStream({
     start(c) { controller = c; },
 
-    async write(chunk) {
+    write(chunk) {
       if (streamWritable.writableNeedDrain || !streamWritable.write(chunk)) {
         backpressurePromise = createDeferredPromise();
         return SafePromisePrototypeFinally(
