@@ -3,7 +3,9 @@
 const {
   ArrayBuffer,
   ArrayBufferPrototypeGetByteLength,
+  ArrayBufferPrototypeGetDetached,
   ArrayBufferPrototypeSlice,
+  ArrayBufferPrototypeTransfer,
   ArrayPrototypePush,
   ArrayPrototypeShift,
   DataView,
@@ -20,7 +22,6 @@ const {
   SafePromiseAll,
   Symbol,
   SymbolAsyncIterator,
-  SymbolDispose,
   SymbolToStringTag,
   TypedArrayPrototypeGetLength,
   Uint8Array,
@@ -50,10 +51,10 @@ const {
 const {
   createDeferredPromise,
   customInspectSymbol: kInspect,
-  isArrayBufferDetached,
   kEmptyObject,
   kEnumerableProperty,
   SideEffectFreeRegExpPrototypeSymbolReplace,
+  SymbolDispose,
 } = require('internal/util');
 
 const {
@@ -103,11 +104,9 @@ const {
   extractHighWaterMark,
   extractSizeAlgorithm,
   lazyTransfer,
-  isViewedArrayBufferDetached,
   isBrandCheck,
   resetQueue,
   setPromiseHandled,
-  transferArrayBuffer,
   nonOpCancel,
   nonOpPull,
   nonOpStart,
@@ -698,7 +697,7 @@ class ReadableStreamBYOBRequest {
     const viewBuffer = ArrayBufferViewGetBuffer(view);
     const viewBufferByteLength = ArrayBufferPrototypeGetByteLength(viewBuffer);
 
-    if (isArrayBufferDetached(viewBuffer)) {
+    if (ArrayBufferPrototypeGetDetached(viewBuffer)) {
       throw new ERR_INVALID_STATE.TypeError('Viewed ArrayBuffer is detached');
     }
 
@@ -725,7 +724,7 @@ class ReadableStreamBYOBRequest {
 
     validateBuffer(view, 'view');
 
-    if (isViewedArrayBufferDetached(view)) {
+    if (ArrayBufferPrototypeGetDetached(view.buffer)) {
       throw new ERR_INVALID_STATE.TypeError('Viewed ArrayBuffer is detached');
     }
 
@@ -1981,7 +1980,7 @@ function readableByteStreamControllerConvertPullIntoDescriptor(desc) {
   if (bytesFilled > byteLength)
     throw new ERR_INVALID_STATE.RangeError('The buffer size is invalid');
   assert(!(bytesFilled % elementSize));
-  const transferredBuffer = transferArrayBuffer(buffer);
+  const transferredBuffer = ArrayBufferPrototypeTransfer(buffer);
 
   if (ctor === Buffer) {
     return Buffer.from(transferredBuffer, byteOffset, bytesFilled / elementSize);
@@ -2650,7 +2649,7 @@ function readableByteStreamControllerPullInto(
 
   let transferredBuffer;
   try {
-    transferredBuffer = transferArrayBuffer(buffer);
+    transferredBuffer = ArrayBufferPrototypeTransfer(buffer);
   } catch (error) {
     readIntoRequest[kError](error);
     return;
@@ -2743,7 +2742,7 @@ function readableByteStreamControllerRespond(controller, bytesWritten) {
       throw new ERR_INVALID_ARG_VALUE.RangeError('bytesWritten', bytesWritten);
   }
 
-  desc.buffer = transferArrayBuffer(desc.buffer);
+  desc.buffer = ArrayBufferPrototypeTransfer(desc.buffer);
 
   readableByteStreamControllerRespondInternal(controller, bytesWritten);
 }
@@ -2793,12 +2792,12 @@ function readableByteStreamControllerEnqueue(controller, chunk) {
   if (closeRequested || stream[kState].state !== 'readable')
     return;
 
-  const transferredBuffer = transferArrayBuffer(buffer);
+  const transferredBuffer = ArrayBufferPrototypeTransfer(buffer);
 
   if (pendingPullIntos.length) {
     const firstPendingPullInto = pendingPullIntos[0];
 
-    if (isArrayBufferDetached(firstPendingPullInto.buffer)) {
+    if (ArrayBufferPrototypeGetDetached(firstPendingPullInto.buffer)) {
       throw new ERR_INVALID_STATE.TypeError(
         'Destination ArrayBuffer is detached',
       );
@@ -2806,7 +2805,7 @@ function readableByteStreamControllerEnqueue(controller, chunk) {
 
     readableByteStreamControllerInvalidateBYOBRequest(controller);
 
-    firstPendingPullInto.buffer = transferArrayBuffer(
+    firstPendingPullInto.buffer = ArrayBufferPrototypeTransfer(
       firstPendingPullInto.buffer,
     );
 
@@ -3104,7 +3103,7 @@ function readableByteStreamControllerRespondWithNewView(controller, view) {
   if (bufferByteLength !== viewBufferByteLength)
     throw new ERR_INVALID_ARG_VALUE.RangeError('view', view);
 
-  desc.buffer = transferArrayBuffer(viewBuffer);
+  desc.buffer = ArrayBufferPrototypeTransfer(viewBuffer);
 
   readableByteStreamControllerRespondInternal(controller, viewByteLength);
 }
