@@ -21,13 +21,11 @@ const {
     ERR_ARG_NOT_ITERABLE,
     ERR_INVALID_ARG_VALUE,
     ERR_INVALID_STATE,
-    ERR_OPERATION_FAILED,
   },
 } = require('internal/errors');
 
 const {
   copyArrayBuffer,
-  detachArrayBuffer,
 } = internalBinding('buffer');
 
 const {
@@ -42,7 +40,6 @@ const {
 } = internalBinding('util');
 
 const assert = require('internal/assert');
-const { isArrayBufferDetached } = require('internal/util');
 
 const {
   validateFunction,
@@ -118,22 +115,6 @@ function isBrandCheck(brand) {
   };
 }
 
-function transferArrayBuffer(buffer) {
-  const res = detachArrayBuffer(buffer);
-  if (res === undefined) {
-    throw new ERR_OPERATION_FAILED.TypeError(
-      'The ArrayBuffer could not be transferred');
-  }
-  return res;
-}
-
-function isViewedArrayBufferDetached(view) {
-  return (
-    ArrayBufferViewGetByteLength(view) === 0 &&
-    isArrayBufferDetached(ArrayBufferViewGetBuffer(view))
-  );
-}
-
 function dequeueValue(controller) {
   assert(controller[kState].queue !== undefined);
   assert(controller[kState].queueTotalSize !== undefined);
@@ -291,11 +272,9 @@ module.exports = {
   invokePromiseCallback,
   isBrandCheck,
   isPromisePending,
-  isViewedArrayBufferDetached,
   peekQueueValue,
   resetQueue,
   setPromiseHandled,
-  transferArrayBuffer,
   nonOpCancel,
   nonOpFlush,
   nonOpPull,
