@@ -22,10 +22,10 @@
 'use strict';
 
 const {
+  ArrayPrototypePush,
   MathTrunc,
   ObjectDefineProperties,
   ObjectDefineProperty,
-  SymbolDispose,
   SymbolToPrimitive,
 } = primordials;
 
@@ -57,6 +57,7 @@ const {
 const {
   promisify: { custom: customPromisify },
   deprecate,
+  SymbolDispose,
 } = require('internal/util');
 let debug = require('internal/util/debuglog').debuglog('timer', (fn) => {
   debug = fn;
@@ -154,7 +155,7 @@ function setTimeout(callback, after, arg1, arg2, arg3) {
       args = [arg1, arg2, arg3];
       for (i = 5; i < arguments.length; i++) {
         // Extend array dynamically, makes .apply run much faster in v6.0.0
-        args[i - 2] = arguments[i];
+        ArrayPrototypePush(args, arguments[i]);
       }
       break;
   }
@@ -224,7 +225,7 @@ function setInterval(callback, repeat, arg1, arg2, arg3) {
       args = [arg1, arg2, arg3];
       for (i = 5; i < arguments.length; i++) {
         // Extend array dynamically, makes .apply run much faster in v6.0.0
-        args[i - 2] = arguments[i];
+        ArrayPrototypePush(args, arguments[i]);
       }
       break;
   }
@@ -296,7 +297,7 @@ function setImmediate(callback, arg1, arg2, arg3) {
       args = [arg1, arg2, arg3];
       for (i = 4; i < arguments.length; i++) {
         // Extend array dynamically, makes .apply run much faster in v6.0.0
-        args[i - 1] = arguments[i];
+        ArrayPrototypePush(args, arguments[i]);
       }
       break;
   }
