@@ -28,7 +28,6 @@ const {
   ArrayPrototypePush,
   FunctionPrototypeBind,
   MathMaxApply,
-  NumberIsFinite,
   NumberIsNaN,
   ObjectDefineProperties,
   ObjectDefineProperty,
@@ -51,7 +50,6 @@ const {
     ERR_ZLIB_INITIALIZATION_FAILED,
   },
   genericNodeError,
-  hideStackFrames,
 } = require('internal/errors');
 const { Transform, finished } = require('stream');
 const {
@@ -71,9 +69,10 @@ const {
 } = require('buffer');
 const { owner_symbol } = require('internal/async_hooks').symbols;
 const {
+  checkRangesOrGetDefault,
   validateFunction,
-  validateNumber,
   validateUint32,
+  validateFiniteNumber,
 } = require('internal/validators');
 
 const kFlushFlag = Symbol('kFlushFlag');
@@ -195,47 +194,6 @@ function zlibOnError(message, errno, code) {
   self[kError] = error;
 }
 
-// 1. Returns false for undefined and NaN
-// 2. Returns true for finite numbers
-// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
-// 4. Throws ERR_OUT_OF_RANGE for infinite numbers
-const checkFiniteNumber = hideStackFrames((number, name) => {
-  // Common case
-  if (number === undefined) {
-    return false;
-  }
-
-  if (NumberIsFinite(number)) {
-    return true; // Is a valid number
-  }
-
-  if (NumberIsNaN(number)) {
-    return false;
-  }
-
-  validateNumber.withoutStackTrace(number, name);
-
-  // Infinite numbers
-  throw new ERR_OUT_OF_RANGE.HideStackFramesError(name, 'a finite number', number);
-});
-
-// 1. Returns def for number when it's undefined or NaN
-// 2. Returns number for finite numbers >= lower and <= upper
-// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
-// 4. Throws ERR_OUT_OF_RANGE for infinite numbers or numbers > upper or < lower
-const checkRangesOrGetDefault = hideStackFrames(
-  (number, name, lower, upper, def) => {
-    if (!checkFiniteNumber.withoutStackTrace(number, name)) {
-      return def;
-    }
-    if (number < lower || number > upper) {
-      throw new ERR_OUT_OF_RANGE.HideStackFramesError(name,
-                                                      `>= ${lower} and <= ${upper}`, number);
-    }
-    return number;
-  },
-);
-
 const FLUSH_BOUND = [
   [ Z_NO_FLUSH, Z_BLOCK ],
   [ BROTLI_OPERATION_PROCESS, BROTLI_OPERATION_EMIT_METADATA ],
@@ -261,7 +219,7 @@ function ZlibBase(opts, mode, handle, { flush, finishFlush, fullFlush }) {
 
   if (opts) {
     chunkSize = opts.chunkSize;
-    if (!checkFiniteNumber(chunkSize, 'options.chunkSize')) {
+    if (!validateFiniteNumber(chunkSize, 'options.chunkSize')) {
       chunkSize = Z_DEFAULT_CHUNK;
     } else if (chunkSize < Z_MIN_CHUNK) {
       throw new ERR_OUT_OF_RANGE('options.chunkSize',
@@ -336,9 +294,12 @@ ObjectDefineProperty(ZlibBase.prototype, 'bytesRead', {
      'This feature will be removed in the future.', 'DEP0108'),
 });
 
+/**
+ * @this ZlibBase
+ * @returns {void}
+ */
 ZlibBase.prototype.reset = function() {
-  if (!this._handle)
-    assert(false, 'zlib binding closed');
+  assert(this._handle, 'zlib binding closed');
   return this._handle.reset();
 };
 
@@ -400,6 +361,10 @@ ZlibBase.prototype.flush = function(kind, callback) {
   }
 };
 
+/**
+ * @this import('stream').Transform
+ * @param {(err?: Error) => any} [callback]
+ */
 ZlibBase.prototype.close = function(callback) {
   if (callback) finished(this, callback);
   this.destroy();
@@ -440,7 +405,7 @@ function processChunkSync(self, chunk, flushFlag) {
   let availOutAfter;
   let availInAfter;
 
-  let buffers = null;
+  const buffers = [];
   let nread = 0;
   let inputRead = 0;
   const state = self._writeState;
@@ -477,10 +442,7 @@ function processChunkSync(self, chunk, flushFlag) {
     if (have > 0) {
       const out = buffer.slice(offset, offset + have);
       offset += have;
-      if (!buffers)
-        buffers = [out];
-      else
-        ArrayPrototypePush(buffers, out);
+      ArrayPrototypePush(buffers, out);
       nread += out.byteLength;
 
       if (nread > self._maxOutputLength) {
@@ -631,12 +593,13 @@ function processCallback() {
   this.cb();
 }
 
+/**
+ * @param {ZlibBase} engine
+ * @private
+ */
 function _close(engine) {
-  // Caller may invoke .close after a zlib error (which will null _handle).
-  if (!engine._handle)
-    return;
-
-  engine._handle.close();
+  // Caller may invoke .close after a zlib error (which will null _handle)
+  engine._handle?.close();
   engine._handle = null;
 }
 
