@@ -11,36 +11,52 @@ using v8::NewStringType;
 using v8::Object;
 using v8::String;
 
-std::vector<std::string> Dotenv::GetPathFromArgs(
+std::vector<Dotenv::env_file_data> Dotenv::GetDataFromArgs(
     const std::vector<std::string>& args) {
+  const std::string_view optional_env_file_flag = "--env-file-if-exists";
+
   const auto find_match = [](const std::string& arg) {
-    return arg == "--" || arg == "--env-file" || arg.starts_with("--env-file=");
+    return arg == "--" || arg == "--env-file" ||
+           arg.starts_with("--env-file=") || arg == "--env-file-if-exists" ||
+           arg.starts_with("--env-file-if-exists=");
   };
-  std::vector<std::string> paths;
-  auto path = std::find_if(args.begin(), args.end(), find_match);
 
-  while (path != args.end()) {
-    if (*path == "--") {
-      return paths;
+  std::vector<Dotenv::env_file_data> env_files;
+  // This will be an iterator, pointing to args.end() if no matches are found
+  auto matched_arg = std::find_if(args.begin(), args.end(), find_match);
+
+  while (matched_arg != args.end()) {
+    if (*matched_arg == "--") {
+      return env_files;
     }
-    auto equal_char = path->find('=');
 
-    if (equal_char != std::string::npos) {
-      paths.push_back(path->substr(equal_char + 1));
+    auto equal_char_index = matched_arg->find('=');
+
+    if (equal_char_index != std::string::npos) {
+      // `--env-file=path`
+      auto flag = matched_arg->substr(0, equal_char_index);
+      auto file_path = matched_arg->substr(equal_char_index + 1);
+
+      struct env_file_data env_file_data = {
+          file_path, flag.starts_with(optional_env_file_flag)};
+      env_files.push_back(env_file_data);
     } else {
-      auto next_path = std::next(path);
+      // `--env-file path`
+      auto file_path = std::next(matched_arg);
 
-      if (next_path == args.end()) {
-        return paths;
+      if (file_path == args.end()) {
+        return env_files;
       }
 
-      paths.push_back(*next_path);
+      struct env_file_data env_file_data = {
+          *file_path, matched_arg->starts_with(optional_env_file_flag)};
+      env_files.push_back(env_file_data);
     }
 
-    path = std::find_if(++path, args.end(), find_match);
+    matched_arg = std::find_if(++matched_arg, args.end(), find_match);
   }
 
-  return paths;
+  return env_files;
 }
 
 void Dotenv::SetEnvironment(node::Environment* env) {
@@ -52,7 +68,7 @@ void Dotenv::SetEnvironment(node::Environment* env) {
 
     auto existing = env->env_vars()->Get(key.data());
 
-    if (existing.IsNothing()) {
+    if (!existing.has_value()) {
       env->env_vars()->Set(
           isolate,
           v8::String::NewFromUtf8(
