@@ -15,6 +15,7 @@
 #include "bindingdata.h"
 #include "cid.h"
 #include "data.h"
+#include "defs.h"
 #include "tokens.h"
 
 namespace node {
@@ -76,10 +77,7 @@ class Packet final : public ReqWrap<uv_udp_send_t> {
          const SocketAddress& destination,
          std::shared_ptr<Data> data);
 
-  Packet(const Packet&) = delete;
-  Packet(Packet&&) = delete;
-  Packet& operator=(const Packet&) = delete;
-  Packet& operator=(Packet&&) = delete;
+  DISALLOW_COPY_AND_MOVE(Packet)
 
   const SocketAddress& destination() const;
   size_t length() const;
